<?php
/**
 * @file
 * hosting_dns hosting_dns.form.inc
 */

/**
 * Configuration form for backup schedules.
 */
function hosting_dns_settings($form, &$form_state) {
  // Todo: Create a form here for default DNS records (and other settings).

  return $form;
}

/**
 * Implements hook_form().
 *
 * This is a node create/edit form for DNS nodes.
 */
function hosting_dns_form($node, &$form_state) {
  $form['#node'] = $node;

  $form['status'] = array(
    '#type' => 'value',
    '#value' => NODE_PUBLISHED,
  );

  // If we are on a node page, pre-fill the site.
  if (arg(0) == 'node' && ($site = node_load(arg(1))) && $site->type == 'site') {
    $form['site'] = array(
      '#type' => 'value',
      '#value' => $site->title,
    );
  }
  else {
    $site = node_load($node->site);

    $form['site'] = array(
      '#type' => 'textfield',
      '#title' => t('Site'),
      '#description' => t('The site to whom this DNS record belongs.'),
      '#default_value' => isset($site->title) ? $site->title : '',
      '#required' => TRUE,
      '#autocomplete_path' => 'hosting/sites/autocomplete',
    );
  }

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('The name of the DNS record.'),
    '#default_value' => isset($node->title) ? $node->title : '',
    '#required' => TRUE,
  );

  $form['dns_ttl'] = array(
    '#type' => 'select',
    '#title' => t('TTL'),
    '#description' => t('The time to live of the DNS record.'),
    '#options' => _hosting_dns_ttl_options(),
    '#default_value' => isset($node->dns_ttl) ? $node->dns_ttl : 86400,
    '#required' => TRUE,
  );

  $form['dns_type'] = array(
    '#type' => 'select',
    '#title' => t('Type'),
    '#description' => t('The type of the DNS record.'),
    '#options' => _hosting_dns_type_options(),
    '#default_value' => isset($node->dns_type) ? $node->dns_type : '',
    '#required' => TRUE,
  );

  $form['dns_value'] = array(
    '#type' => 'textfield',
    '#title' => t('Value'),
    '#description' => t('The value of the DNS record.'),
    '#default_value' => isset($node->dns_value) ? $node->dns_value : '',
    '#required' => TRUE,
  );

  return $form;
}

/**
 * Implements hook_form_alter().
 */
function hosting_dns_form_alter(&$form, &$form_state, $form_id) {
  if ($form_id == 'dns_node_form') {
    // Hide the vertical tabs from the DNS creation form.
    $form['additional_settings']['#access'] = FALSE;

    // Suppress the preview options as it makes no sense.
    $form['actions']['preview']['#access'] = FALSE;
  }
}

/**
 * Implements hook_validate().
 */
function hosting_dns_validate($node, $form, &$form_state) {

  // Make sure the DNS name is lowercase.
  $node->title = strtolower(trim($node->title));

  if (preg_match("/^[a-z0-9-._*]*$/", $node->title) === 0) {
    form_set_error('title', t('This is not a valid DNS name, it can only contain letters, numbers, asterisks (*), dashes (-), dots (.) and underscores (_).'));
  }

  // Check if a valid IPv4 address is specified for the A record.
  if ($node->dns_type == 'A' && filter_var($node->dns_value, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4) === FALSE) {
    form_set_error('dns_value', t('This is not a valid IPv4 address, and cannot be used as an A record.'));
  }

  // Check if a valid IPv6 address is specified for the AAAA record.
  if ($node->dns_type == 'AAAA' && filter_var($node->dns_value, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6) === FALSE) {
    form_set_error('dns_value', t('This is not a valid IPv6 address, and cannot be used as an A record.'));
  }
}

/**
 * Update TTL VBO action form.
 *
 * @see hosting_dns_update_ttl_submit()
 * @see hosting_dns_update_ttl()
 */
function hosting_dns_update_ttl_form($settings, &$form_state) {
  $form['dns_ttl'] = array(
    '#type' => 'select',
    '#title' => t('TTL'),
    '#description' => t('The TTL of the DNS record.'),
    '#options' => _hosting_dns_ttl_options(),
    '#required' => TRUE,
  );

  return $form;
}

/**
 * Update TTL VBO action form submit function.
 *
 * @see hosting_dns_update_ttl_form()
 * @see hosting_dns_update_ttl()
 */
function hosting_dns_update_ttl_submit($form, &$form_state) {
  $context['ttl'] = $form_state['values']['dns_ttl'];
  return $context;
}
