<?php

/**
 * @file
 * DNS nodeapi implementations.
 */

/**
 * Implements hook_node_info().
 */
function hosting_dns_node_info() {
  $types['dns'] = array(
    'type' => 'dns',
    'name' => t('DNS record'),
    'base' => 'hosting_dns',
    'module' => 'hosting_dns',
    'has_title' => TRUE,
    'title_label' => t('Name'),
    'description' => t("<strong>A single DNS record for a site.</strong>\nMultiple of these records can exist on a site, and combined they form a DNS zone."),
    'has_body' => 0,
    'body_label' => '',
    'min_word_count' => 0,
  );
  return $types;
}

/**
 * Implements hook_node_access().
 */
function hosting_dns_node_access($node, $op, $account) {
  if (user_access('administer hosting dns', $account)) {
    return NODE_ACCESS_ALLOW;
  }
}

/**
 * Implements hook_entity_property_info().
 */
function hosting_dns_entity_property_info_alter(&$info) {
  $dns_properties = &$info['node']['bundles']['dns']['properties'];

  $dns_properties['site'] = array(
    'label' => t('Site'),
    'description' => t('The site that this zone belongs to.'),
    'type' => 'node',
    'bundle' => 'site',
  );

  $dns_properties['dns_ttl'] = array(
    'label' => t('TTL'),
    'description' => t('The TTL of this record.'),
    'type' => 'integer',
  );

  $dns_properties['dns_type'] = array(
    'label' => t('Type'),
    'description' => t('The type of this record.'),
    'type' => 'text',
  );

  $dns_properties['dns_value'] = array(
    'label' => t('Value'),
    'description' => t('The value of this record.'),
    'type' => 'text',
  );

  return $info;
}

/**
 * Implements hook_view().
 */
function hosting_dns_view($node, $view_mode, $langcode) {
  // If the user attempts to view a DNS node,
  // we redirect him to the DNS overview page.
  if ($node->type == 'dns') {
    drupal_goto('node/' . $node->site . '/dns');
  }
}

/**
 * Implements hook_delete().
 */
function hosting_dns_delete($node) {
  db_delete('hosting_dns_records')
    ->condition('nid', $node->nid)
    ->execute();
}

/**
 * Implements hook_insert().
 */
function hosting_dns_insert(&$node) {

  $result = db_query("SELECT s.nid FROM {hosting_site} s JOIN {node} n ON s.nid = n.nid WHERE n.title = :title AND n.type = 'site'", array(":title" => $node->site))->fetchField();
  $site = isset($result) ? node_load($result) : NULL;
  $node->site = isset($site) ? $site->nid : '';

  $id = db_insert('hosting_dns_records')
    ->fields(array(
      'vid' => $node->vid,
      'nid' => $node->nid,
      'site' => $node->site,
      'dns_ttl' => $node->dns_ttl,
      'dns_type' => $node->dns_type,
      'dns_value' => $node->dns_value,
    ))
    ->execute();
}

/**
 * Implements hook_update().
 */
function hosting_dns_update(&$node) {
  // If this is a new node or we're adding a new revision.
  if (!empty($node->revision)) {
    hosting_dns_insert($node);
  }
  else {
    db_update('hosting_dns_records')
      ->fields(array(
        'site' => $node->site,
        'dns_ttl' => $node->dns_ttl,
        'dns_type' => $node->dns_type,
        'dns_value' => $node->dns_value,
      ))
      ->condition('vid', $node->vid)
      ->execute();
  }
}

/**
 * Implements hook_load().
 */
function hosting_dns_load($nodes) {
  foreach ($nodes as $nid => &$node) {
    $additions = db_query('SELECT site, dns_ttl, dns_type, dns_value FROM {hosting_dns_records} WHERE vid = :vid', array(':vid' => $node->vid))->fetch();
    foreach ($additions as $property => &$value) {
      $node->$property = $value;
    }
  }
}
